


// MARK: - painter funcs

function fillPainter(color)
{
    return {
        nsClass: "GXPainter",
        gxFillColor: color,
    };
}

function fillPainterWithSize(color, size)	// size expected as string, eg. "10, 20"
{
    return {
        nsClass: "GXPainter",
        gxFillColor: color,
		gxDefaultSize: size,
    };
}

function roundRectPainter(color, radius)
{
    return {
        nsClass: "GXPainter",
        gxFillColor: color,
        gxCornerRadius: radius,
    };
}


// MARK: - view funcs

function labelView(text, styles, valueKey)
{
	if (valueKey == null)
		valueKey = "";
	
    return {
        "+": "LabelView.right.size13." + styles,
        gxDefaultValue: text,
		gxValueKey: valueKey,
    };
}

function spacerView()
{
    return {
        nsClass: "GXDrawView",
        gxHidden: true,
    };
}

function filledView(color)
{
    return {
        nsClass: "GXDrawView",
        gxBackPainter: fillPainter(color),
    };
}

function groupTitleView(title, styles)
{
	var template = "GroupTitleView";
	if (styles != null)
		template = "GroupTitleView." + styles;
	
	return {
		"+": template,
		gxDefaultValue: title,
	};
}


// MARK: - formatter funcs

function disclosureAxFormatter(title)
{
	return {
		nsClass: "GXDictionaryFormatter",
		gxDictionary: {
			0: title + ", collapsed",
			1: title + ", expanded",
		}
	};
}


// MARK: - menu funcs

function menuItem(title, repValue, styles, displayValue)
{
	return {
		"+": "MenuItem." + styles,
		gxTitle: title,
		gxDisplayValue: displayValue ? displayValue : title,	// displayed by popupmenu when selected
		gxValue: repValue,
	};
}

function menuDivider()
{
	return {
		"+": "MenuItem",
		gxIsSeparator: true,
	};
}


// MARK: - experiments (mostly unused)

Object.defineProperty( Object.prototype, "insets",
{ enumerable: false,
  value: function(insets)
          {
              var result = Object.create(this);
              result.gxEdgeInsets = insets;
              return result;
          }
});

// function to strip a key from an object/dictionary
Object.defineProperty( Object.prototype, "removeProperty",
{ enumerable: false,
  value: function(propertyKey)
		  {
			  console.log(JSON.stringify(this));
			  var result = Object.assign(this, {propertyKey: null});
			  console.log(JSON.stringify(result));
			 // result[propertyKey] = null;
			 // console.log(JSON.stringify(result));
			  return result;
		  }
});

Object.defineProperty( Object.prototype, "displayValue",
{ enumerable: false,
  value: function(displayVal)
		  {
			  var result = Object.create(this);
			  result.gxDisplayValue = displayVal;
			  return result;
		  }
});

Object.defineProperty( Object.prototype, "icon",
{ enumerable: false,
  value: function(iconName)
		  {
			  var result = Object.create(this);
			  result.gxIcon = iconName;
			  return result;
		  }
});

Object.defineProperty( Object.prototype, "replaceKeyPathValue",
{ enumerable: false,
  value: function()
		  {
			  // arguments expected are eg: key, key, key, value
			  // like a key path, followed by the value to set at the leaf
			  
			  // must at least have one key + value
			  var length = arguments.length;
			  if (length < 2)
				  return;
	
			  var object = this;
			  for (var i = 0; i < length - 2; i++)
			  {
				  object = object[arguments[i]];
			  }
			  
			  // set value
			  object[arguments[length - 2]] = arguments[length - 1];

			  return this;
		  }
});

function enclosureView(viewName, subViewMarkup)
{
	return {
		nsClass: "GXDrawView",
		gxHideOnNilValue: true,
		gxLayout: [
			"H:|-0-[self]-0-|",
			"V:[self(34)]"
		],
		gxViewName: viewName,	// viewName is shared with enclosure, so it hides when value is nil (auto value lookup)
		gxChildViews:
		{
			[viewName]: subViewMarkup,
		},
	};
}

function presetsGroup()
{
	return {
		nsClass: "GXDrawView",
		gxLayout: [
			"H:[self]-8-|",
			"V:[self]-8-|",
			"H:|-4-[presetsLabel(55)]-8-[presets(202)]-4-|&alignBaseline",
			"V:|-4-[presets(25)]-4-|"
		],
		gxChildViews:
		{
			presets:
			{
				"+": "PresetsPopupMenu"
			},
			presetsLabel: labelView("Presets:", "right")
		}
	};
}

