

function roundRectPainterWithOffet(color, radius, offset = [0,0])
{
    return {
        nsClass: "GXPainter",
        gxFillColor: color,
        gxDefaultSize: `{${radius},${radius}`,
        gxEdgeInsets: `${-offset[0] * -1},${-offset[1]},${-offset[0]},${-offset[1] * -1}`,
        gxCornerRadius: radius,
    };
}

markup.push({

// MARK: - EQ block

ParametricEQBlock:
{
	"+": "BlockTemplate",
	gxPopoverInfo: "ParametricEQPopover", 
	gxTitle: "Parametric EQ",
	gxLibraryDescription: "Fine-tune specific frequencies",
	geNodeClass: "AHNBandEQNode",
	h4PresetsId: "ParametricEQPresets",
	gxIconPainter: "ImagePainter.block.10band",
	gxLibraryIcon: "library.10band",
},

// MARK: - EQ popover
    
ParametricEQPopover:
{
	"+": "PopoverTemplate",
	gxTitle: "Parametric Equalizer", 
	gxLayout: [
		"H:|-12-[self]-12-|",
		"V:|-12-[self]-12-|",
		"H:|-0-[group]-0-|",
		"V:|-41-[group]-8-[presetsGroup]"
	],
	gxCanDrawSubviewsIntoLayer: true,
	gxWantsLayer: true,
	gxChildViews:
	{
		title: "PopoverTitle",
		switch: "PopoverOnOffSwitch",
		presetsGroup: presetsGroup(),
		group: {
			"+": "GroupView",
			gxBackPainter: null,
			gxLayout: [ 
				"V:|[Low]|", 
				"H:|-20-[preamp]-10-[meter(40)]-10-[Low]-10-[LowMid]-10-[Mid]-10-[HighMid]-10-[High]-20-|&alignCenterY", 
				"V:|[meter]|",
			],
			gxChildViews: [
				RoundKnob("preamp", 50, -24, 24, "ReadoutDB", "node.preampDB", detents = true), 
				EQLevelMeter("meter",0),
				BandGroup("Low", 0, 20, 500),
				BandGroup("Low-Mid", 1, 200, 2000),
				BandGroup("Mid", 2, 500, 4000),
				BandGroup("High-Mid", 3, 2000, 10000),
				BandGroup("High", 4, 5000, 20000), 
			]
		}
	}
}, 
 
// MARK: - EQ presets

ParametricEQPresets: // name tied to saved presets and AH3 import
{
	nsClass: "GEPresetsGroup",
	gxTitle: "Parametric EQ Presets",
	gxDefaultPreset: "Off (Flat)",
	gxPresetValueKeys: ["node.profileSettings"], 
	gxPresets: [
	{ 
		presetName: "Off (Flat)",	
		presetValues: { 
			"node.profileSettings" : {
				"preamp" : 0, 
				"bands" : [ EQBandValues(0,1,34), EQBandValues(0,2,200), EQBandValues(0,2,1000), EQBandValues(0,2,5000), EQBandValues(0,3,12500) ] 
			}
		}  
	},],
}, 

EQTabBarButton: {
	"+": "TabButtonView", 
	gxIntrinsicHeight: 20, 
},

"ReadoutHertz": {
	"+": "ReadoutView.centerx",
	gxFormatter: { "+" : "HertzFormatter" },
},

"ReadoutDB": {
	"+": "ReadoutView.centerx",
	gxFormatter: { "+" : "DBFormatter" },
},

"ReadoutQ": {
	"+": "ReadoutView.centerx", 
	gxFormatter: { nsClass: "GXScaledFormatter", format: "% .1f" },
},

})

function BandGroup(name, band, freqL, freqH)
{
	return {
		"+": "GroupView", 
		gxBackPainter: null,
		gxViewName: name.replace("-", ""),
		gxLayout: [
			"H:|-5-[gain]-(>=25)-[freq]-5-|&alignCenterY",
			"V:[self]-(<=0)-[q]&alignCenterX",
			"H:[gain]-(>=0)-[q]-(>=0)-[freq]&alignTop",
			"V:|-50-[gain]-25-[EQTypes]-5-|",
			"V:|-5-[power]",
			"H:|-5-[power]-5-[bandLabel]&alignCenterY",
			"H:|[EQTypes]|",
		],
		gxChildViews : [
			{ "+" : "LabelView.left.size12.boldWeight", gxViewName: "bandLabel", gxDefaultValue: name },
			PowerButton("power", `node.b_enabled${band}`),
			RoundKnob("gain", 40, -24, 24, "ReadoutDB", `node.b_gain${band}`, true),
			RoundKnob("q", 24, 0.5, 10, "ReadoutQ", `node.b_width${band}`, false, 1.4),
			RoundKnob("freq", 40, freqL, freqH, "ReadoutHertz", `node.b_freq${band}`),   
			{ 
				nsClass: "GXDrawView",
				gxViewName: "EQTypes",
				gxLayout: ["H:|-0-[HP(25)]-0-[LS(==HP)]-0-[PK(==HP)]-0-[HS(==HP)]-0-[LP(==HP)]-0-|", "V:|[HP(20)]|"],
				gxChildViews: [
					{ "+": "EQTabBarButton.leftSegment", 	gxViewName: "HP", gxForePainter: { "+" : "ImagePainter", gxImage: "peq-hp" }, gxValueKey: `node.b_type${band}_x_0` },
					{ "+": "EQTabBarButton.middleSegment", 	gxViewName: "LS", gxForePainter: { "+" : "ImagePainter", gxImage: "peq-ls" }, gxValueKey: `node.b_type${band}_x_1` },
					{ "+": "EQTabBarButton.middleSegment", 	gxViewName: "PK", gxForePainter: { "+" : "ImagePainter", gxImage: "peq-pk" }, gxValueKey: `node.b_type${band}_x_2` },
					{ "+": "EQTabBarButton.middleSegment", 	gxViewName: "HS", gxForePainter: { "+" : "ImagePainter", gxImage: "peq-hs" }, gxValueKey: `node.b_type${band}_x_3` },
					{ "+": "EQTabBarButton.rightSegment", 	gxViewName: "LP", gxForePainter: { "+" : "ImagePainter", gxImage: "peq-lp" }, gxValueKey: `node.b_type${band}_x_4` }
				]
			}
		]
	}	
} 

function EQLevelMeter(name)
{
	return { 
		nsClass: "GXLayerHostingView",   
		gxViewModel: { nsClass: "AH4LevelMeterVModel" },
		gxViewName:name,
		gxValueKey: "",
		gxLayerZPosition: 2,
		gxEdgeInsets: "1, 1, 1, 1",
		gxLayout: [
				    
			"V:|-4-[leftClip(9)]-4-[leftPeak]-4-|",
			"V:[leftClip]-4-[leftRMS]-4-|",
			"V:[leftClip]-4-[leftHold]-4-|",
				   
			"H:|-5-[leftPeak(9)]",
			"H:|-5-[leftRMS(9)]",
			"H:|-5-[leftHold(9)]",
			   "H:|-5-[leftClip(9)]",

			"V:|-4-[rightClip(9)]-4-[rightPeak]-4-|",
			"V:[rightClip]-4-[rightRMS]-4-|",
			"V:[rightClip]-4-[rightHold]-4-|",
			
			"H:[rightPeak(9)]-5-|",
			"H:[rightRMS(9)]-5-|",
			"H:[rightHold(9)]-5-|",
			"H:[rightClip(9)]-5-|",
		],
		gxRootLayer:
		{
			nsClass: "GXDrawLayer",
			gxLayerZPosition: 10,
			gxSublayers:
			{
				leftPeak: { "+": "PeakLayer.blockface" },
				leftRMS: { "+": "RMSLayer.blockface" },
				leftClip: { "+": "ClipLayer.blockface" },
				leftHold: { "+": "HoldLayer.blockface" },

				rightPeak: { "+": "PeakLayer.blockface" },
				rightRMS: { "+": "RMSLayer.blockface" },
				rightClip: { "+": "ClipLayer.blockface" },
				rightHold: { "+": "HoldLayer.blockface" },
			}
		}
	}
}

function RoundKnob(name, size, low, high, readoutTemplate, keyPath, detents = false, defaultVal = null)
{
	defaultVal = defaultVal ? defaultVal : (low + (high - low) / 2.0);
	return {
		nsClass: "GXKnobView", 
		gxViewName: name,
		gxLayout: [`H:[self(${size})]`, "H:[selfReadout(60)]", `V:|-(>=0)-[selfLabel]-(>=0)-[self(${size})]-(5)-[selfReadout]&alignCenterX`],
		gxViewFlipped: false,
		gxIgnoreDoubleClick: false,
		gxDetents: `${low},${defaultVal},${high}`,
		gxValueKey: keyPath,
		gxIntrinsicWidth: size,
		gxIntrinsicHeight: size, 
		gxRangeMin: low,
		gxRangeMax: high,
		gxDefaultValue: defaultVal,
		gxDisableOnKey: `${keyPath}disabled`,
		gxDetentSnapDistance: detents ? 8 : null, 
		gxKnobPainter: roundRectPainterWithOffet("ColorTheme1", size), // painter for whole knob
		gxKnobPainterExtra: roundRectPainterWithOffet("ColorTheme3", 6, [(-(size * 0.5)) + 5,0]), // painter for the knob marking
		gxLabelTemplate: "LabelView.left.size10",
		gxLabelText: name, 
		//gxReadoutTemplate: readoutTemplate,
		gxReadoutKeyPath: keyPath,
		axIsIgnored: false,
	}
}

function PowerButton(name, keyPath)
{
	return {
		"+" : "ButtonView",
		gxViewName: name,  
		gxTitle: "\u{23FB}", // power symbol
		gxCheckBoxRole: true,
		gxValueKey: keyPath,
		gxLayout: ["H:[self(20)]","V:[self(20)]"], 
	}
}

function presetWithDictionaries(dictionaries)
{
	var presets = {}
	for (var dict in dictionaries)
	{
		presets[dict.key] = dictionaries[dict.key]
	}
	return presets
}

function EQBandValues(gain, type, freq, width = 1.4)
{
	return { "enabled": true, "freq": freq, "gain" : gain, "q": width, "type" : type  };
} 
